==> 0.6.0.tar.gz: lib/src/event/room/room_event.dart <==
// Copyright (C) 2019  Wilko Manger
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/message_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/user.dart';
import 'package:meta/meta.dart';

import 'redaction_event.dart';
import 'state/avatar_change_event.dart';
import 'state/member_change_event.dart';
import 'state/name_change_event.dart';

abstract class RoomEvent extends Event {
  final Room room;
  final User sender;
  final DateTime time;

  @override
  final SentState sentState;

  RoomEvent(RoomEventArgs args)
    : room = args.room,
      sender = args.sender,
      time = args.time,
      sentState = args.sentState,
      super(args.id, args.unsigned);

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'sender': sender.id.toString(),
      'origin_server_ts': time.millisecondsSinceEpoch
    });

  /// Create event from the given [json].
  static Future<RoomEvent> fromJson(
    RoomEventArgs args, Map<String, dynamic> json, {ProcessContext context}) async {
    final type = json['type'];
    final stateKey = json['state_key'];

    final finalArgs = await args.parse(
      json: json,
      context: context
    );

    RoomEvent event;
    switch (type) {
      case NameChangeEvent.type:
        event = NameChangeEvent.fromJson(finalArgs, json);
        break;
      case AvatarChangeEvent.type:
        event = AvatarChangeEvent.fromJson(finalArgs, json);
        break;
      case MemberChangeEvent.type:
        event = MemberChangeEvent.fromJson(
          finalArgs,
          stateKey,
          json
        );
        break;
      case MessageEvent.type:
        event = MessageEvent.fromJson(finalArgs, json);
        break;
      case RedactionEvent.type:
        event = RedactionEvent.fromJson(finalArgs, json);
        break;
    }

    if (event != null && json.containsKey('unsigned')
        && json['unsigned']?.containsKey('redacted_because') == true) {
      final redactedBecause = json['unsigned']['redacted_because'];
      if (redactedBecause != null) {
        final redaction = RedactionEvent.fromJson(
          await finalArgs.parse(
            json: redactedBecause,
            context: context
          ),
          redactedBecause
        );

        final redacted = RedactedEvent(
          redaction: redaction,
          original: event
        );

        event = redacted;
      }
    }

    if (event != null) {
      context.timeline[event.id] = event;
    }

    if (event is MemberChangeEvent) {
      final user = User(
        id: event.sender.id,
        state: UserState(
          roomId: event.room.id,
          since: event.time,
          displayName: event.content.displayName,
          avatarUrl: event.content.avatarUrl
        )
      );

      context.setUser(user);
    }

    return event;
  }
}

class RoomEventArgs {
  final EventId id;
  final Room room;
  final User sender;
  final DateTime time;
  final UnsignedData unsigned;
  final SentState sentState;

  RoomEventArgs({
    this.id,
    this.room,
    this.sender,
    this.time,
    this.unsigned,
    this.sentState = SentState.sent
  });

  RoomEventArgs copyWith({
    EventId id,
    Room room,
    User sender,
    DateTime time,
    UnsignedData unsigned,
    SentState sentState
  }) => RoomEventArgs(
    id: id ?? this.id,
    room: room ?? this.room,
    sender: sender ?? this.sender,
    time: time ?? this.time,
    unsigned: unsigned ?? this.unsigned,
    sentState: sentState ?? this.sentState
  );

  Future<RoomEventArgs> parse({
    @required Map<String, dynamic> json,
    ProcessContext context,
    EventId id,
    Room room,
    User sender,
    DateTime time,
    UnsignedData unsigned,
    SentState sentState
  }) async {
    // These might be null
    id = id ?? EventId(json['event_id']);

    room = this.room ?? room;

    final originServerTs = json['origin_server_ts'];
    if (originServerTs != null) {
      time = DateTime.fromMillisecondsSinceEpoch(originServerTs);
    }

    final senderId = UserId(json['sender']);

    final contextIsNull = context == null;

    sender =
      this.sender
      ?? (!contextIsNull ? context.currentUsers[senderId] : null)
      ?? await room?.members?.get(senderId, at: time)
      ?? User(
        id: senderId,
        state: UserState(
          roomId: room.id,
          since: time
          ),
        );


    var unsignedJson = json['unsigned'];
    if (unsigned == null && unsignedJson != null) {

      final transactionId = json['unsigned']['transaction_id'];

      if (transactionId != null) {
        unsigned = UnsignedData(transactionId: transactionId);
      } else {
        unsigned = null;
      }
    }

    return this.copyWith(
      id: id,
      room: room,
      sender: sender,
      time: time,
      unsigned: unsigned,
      sentState: sentState
    );
  }
}

==> 0.9.0.tar.gz: lib/src/event/room/room_event.dart <==
// Copyright (C) 2019  Wilko Manger
// Copyright (C) 2019  Mathieu Velten
//
// This file is part of matrix-dart-sdk.
//
// matrix-dart-sdk is free software: you can redistribute it and/or modify
// it under the terms of the GNU Affero General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// matrix-dart-sdk is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Affero General Public License for more details.
//
// You should have received a copy of the GNU Affero General Public License
// along with matrix-dart-sdk.  If not, see <https://www.gnu.org/licenses/>.

import 'package:matrix_sdk/src/event/event.dart';
import 'package:matrix_sdk/src/event/room/message_event.dart';
import 'package:matrix_sdk/src/identifier.dart';
import 'package:matrix_sdk/src/room/room.dart';
import 'package:matrix_sdk/src/user.dart';
import 'package:meta/meta.dart';

import 'redaction_event.dart';
import 'state/avatar_change_event.dart';
import 'state/member_change_event.dart';
import 'state/name_change_event.dart';

abstract class RoomEvent extends Event {
  final Room room;
  final User sender;
  final DateTime time;

  @override
  final SentState sentState;

  RoomEvent(RoomEventArgs args)
    : room = args.room,
      sender = args.sender,
      time = args.time,
      sentState = args.sentState,
      super(args.id, args.unsigned);

  @override
  Map<String, dynamic> toJson() => super.toJson()
    ..addAll({
      'sender': sender.id.toString(),
      'origin_server_ts': time.millisecondsSinceEpoch
    });

  /// Create event from the given [json].
  static Future<RoomEvent> fromJson(
    RoomEventArgs args, Map<String, dynamic> json, {ProcessContext context}) async {
    final type = json['type'];
    final stateKey = json['state_key'];

    final finalArgs = await args.parse(
      json: json,
      context: context
    );

    RoomEvent event;
    switch (type) {
      case NameChangeEvent.type:
        event = NameChangeEvent.fromJson(finalArgs, json);
        break;
      case AvatarChangeEvent.type:
        event = AvatarChangeEvent.fromJson(finalArgs, json);
        break;
      case MemberChangeEvent.type:
        event = MemberChangeEvent.fromJson(
          finalArgs,
          stateKey,
          json
        );
        break;
      case MessageEvent.type:
        event = MessageEvent.fromJson(finalArgs, json);
        break;
      case RedactionEvent.type:
        event = RedactionEvent.fromJson(finalArgs, json);
        break;
    }

    if (event != null && json.containsKey('unsigned')
        && json['unsigned']?.containsKey('redacted_because') == true) {
      final redactedBecause = json['unsigned']['redacted_because'];
      if (redactedBecause != null) {
        final redaction = RedactionEvent.fromJson(
          await finalArgs.parse(
            json: redactedBecause,
            context: context
          ),
          redactedBecause
        );

        final redacted = RedactedEvent(
          redaction: redaction,
          original: event
        );

        event = redacted;
      }
    }

    if (event != null && context != null) {
      context.timeline[event.id] = event;
    }

    if (event is MemberChangeEvent) {
      context?.setUser(event.content.subject);
    }

    return event;
  }
}

class RoomEventArgs {
  final EventId id;
  final Room room;
  final User sender;
  final DateTime time;
  final UnsignedData unsigned;
  final SentState sentState;

  RoomEventArgs({
    this.id,
    this.room,
    this.sender,
    this.time,
    this.unsigned,
    this.sentState = SentState.sent
  });

  RoomEventArgs copyWith({
    EventId id,
    Room room,
    User sender,
    DateTime time,
    UnsignedData unsigned,
    SentState sentState
  }) => RoomEventArgs(
    id: id ?? this.id,
    room: room ?? this.room,
    sender: sender ?? this.sender,
    time: time ?? this.time,
    unsigned: unsigned ?? this.unsigned,
    sentState: sentState ?? this.sentState
  );

  Future<RoomEventArgs> parse({
    @required Map<String, dynamic> json,
    ProcessContext context,
    EventId id,
    Room room,
    User sender,
    DateTime time,
    UnsignedData unsigned,
    SentState sentState
  }) async {
    // These might be null
    id = id ?? EventId(json['event_id']);

    room = this.room ?? room;

    final originServerTs = json['origin_server_ts'];
    if (originServerTs != null) {
      time = DateTime.fromMillisecondsSinceEpoch(originServerTs);
    }

    final senderId = UserId(json['sender']);

    final contextIsNull = context == null;

    sender =
      this.sender
      ?? (!contextIsNull ? context.currentUsers[senderId] : null)
      ?? await room?.members?.get(senderId, at: time)
      ?? User(
        id: senderId,
        state: UserState(
          roomId: room.id,
          since: time
          ),
        );


    var unsignedJson = json['unsigned'];
    if (unsigned == null && unsignedJson != null) {

      final transactionId = json['unsigned']['transaction_id'];

      if (transactionId != null) {
        unsigned = UnsignedData(transactionId: transactionId);
      } else {
        unsigned = null;
      }
    }

    return this.copyWith(
      id: id,
      room: room,
      sender: sender,
      time: time,
      unsigned: unsigned,
      sentState: sentState
    );
  }
}
